package EPCFilter;

import RFIDReader.OnTagReadResponse;
import RFIDReader.RFIDReader;
import RFIDReader.TagMask;
import RFIDReader.TagReadInfo;

/*
 * The following code is an example of how to set an EPC filter and read tags from
 * with a connected reader. To run the code successfully the IP Address must be
 * changed to match that of the reader for intended use.
 */

public class EPCFilter {
	static final String DEFAULT_READER_IP_ADDRESS = "192.168.1.120";
	static final int PORT_NUMBER = 5000;
	static final int maxResponseSize = 1000;
	static final String[] TAG_EPC_FILTER = {"E200","63B6","0720"};
	static RFIDReader reader = null;
	public static boolean reading = false;
	
	public static void main(String[] args){
		try
		{
			String readerIPAddress = DEFAULT_READER_IP_ADDRESS;
			int PortNumber = PORT_NUMBER;

			System.out.println("Setting up ReaderComm instance on IP Address " + readerIPAddress);
			
			reader = new RFIDReader(readerIPAddress, PortNumber);
			
			
			TagReadEPCFilter(0);
			EPCFilter.ReadTest();
			EPCFilter.reader.ClearInventoryFilter(true);
			
		}
		catch (Exception ex)
		{
			System.out.println("Main: " + ex.toString());
		}
	}
	
	public static void TagReadEPCFilter(int startAddress) throws Exception
	{
		TagMask tagMask = new TagMask();
		
		String[] tagEPCFilter = TAG_EPC_FILTER;
		tagMask.maskBank = 1;
		tagMask.maskStartBitAddress = startAddress + 32;
		tagMask.maskBitLength = tagEPCFilter.length * 16;
		
		
		for (int idx = 0; idx < tagMask.maskBitLength / 16; idx++)
		{
			short epcWord = 0;
			epcWord = (short)Integer.parseInt(tagEPCFilter[idx],16);
			tagMask.mask[idx * 2] = (byte)((epcWord >> 8) & 0xFF);
			tagMask.mask[idx * 2 + 1] = (byte)(epcWord & 0xFF);
		}
		EPCFilter.reader.SetInventoryFilter(tagMask);
	}
	
	public static void ReadTest() throws Exception, InterruptedException
	{
		TagReadHandler tagReadHandler = new TagReadHandler();
		EPCFilter.reader.SetTagReadCallback(tagReadHandler);
		System.out.println("Starting Continuous Read ...");
		EPCFilter.reading = true;
		EPCFilter.reader.StartContinuousRead();
		
		// Read for 5 seconds then issue stop command
		Thread.sleep(5000);
		
		System.out.println("Stopping Continuous Read ...");
		EPCFilter.reader.StopContinuousRead();
		
		// Sleep for an additional 2 seconds to read queued up
		// tag data and to wait for stop command response.
		Thread.sleep(2000);
	}
}
class TagReadHandler implements OnTagReadResponse
{
	public void handleTagRead(TagReadInfo readInfo)
	{
		if (readInfo.tagReadStatus == TagReadInfo.OK)
			System.out.println(readInfo.EPC);
		else if (readInfo.tagReadStatus == TagReadInfo.DONE)
		{
			EPCFilter.reading = false;
			System.out.println("Received Stop Response");
		}
	}
}
