package ReadWriteTags;

import RFIDReader.RFIDReader;
import RFIDReader.TagFields;
import RFIDReader.TagMask;

/*
 * The following code is an example of how to read/write a tags data using the
 * connected reader. To run the code successfully, the IP Address must be changed
 * to match that of the reader for intended use, there should be no need to change
 * the tag access or kill password unless the tag has been previously locked.
 */

public class ReadWriteTags {
	
	static final String DEFAULT_READER_IP_ADDRESS = "192.168.1.120";
	static final int PORT_NUMBER = 5000;
	static final int maxResponseSize = 1000;
	static RFIDReader reader = null;
	static final int TAG_ACCESS_PASSWORD = 0x00000000;	
	static final int TAG_KILL_PASSWORD = 0x000000000;
	static final String[] TAG_EPC = {"5555","5555","5555","5555","5555","5555"};
	
	private static String[] TAG_ID = {"", "", "", "", "", ""};
	
	public static void main(String[] args){
		try
		{
			String readerIPAddress = DEFAULT_READER_IP_ADDRESS;
			int PortNumber = PORT_NUMBER;

			System.out.println("Setting up ReaderComm instance on IP Address " + readerIPAddress);
			
			reader = new RFIDReader(readerIPAddress, PortNumber);
			
			ReadTagID(TAG_ACCESS_PASSWORD);
			ReadTagData(TAG_ACCESS_PASSWORD);
			WriteTagData(TAG_ACCESS_PASSWORD);
			ReadTagData(TAG_ACCESS_PASSWORD);
			WriteTagData2(TAG_ACCESS_PASSWORD); // reset the tag data.
			
			

		}
		catch (Exception ex)
		{
			System.out.println("Main: " + ex.toString());
		}
	}
	
	public static void ReadTagID(int accessPassword) throws Exception
	{
		TagFields tagFields = new TagFields();
		TagMask tagMask = new TagMask();
		byte[] dataBanks = { 0, 0, 1, 2, 3 };
		short[] dataBankLengths = { 2,     // Kill Password
									2,     // Access Password
									8,     // EPC length
									6,     // TID length
									8 };   // User data length
		short[] dataBankAddress = { 0, 2, 0, 0, 0 };

		tagMask.maskBank = 1;
		tagMask.maskStartBitAddress = 32;
		tagMask.maskBitLength = 96;
		String[] tagEPC = TAG_EPC;
		
		for (int idx = 0; idx < tagMask.maskBitLength / 16; idx++)
		{
			short epcWord = 0;
			epcWord = (short)Integer.parseInt(tagEPC[idx],16);
			tagMask.mask[idx * 2] = (byte)((epcWord >> 8) & 0xFF);
			tagMask.mask[idx * 2 + 1] = (byte)(epcWord & 0xFF);
		}

		
			tagFields.errorCode = 0;
			tagFields.data = new short[100];
			tagFields.dataWordLength = dataBankLengths[3];
			tagFields.dataStartWord = dataBankAddress [2];

			tagFields.dataBank = dataBanks[3];

			ReadWriteTags.reader.ReadTagData(accessPassword, tagMask, tagFields);
			
			//String tidTag = "";
			
			for (int jdx = 0; jdx < tagFields.dataWordLength; jdx++)
			{
				String tidWord = String.format("%04X", tagFields.data[jdx]);
				TAG_ID[jdx] += tidWord; 
			}
			System.out.printf("TID: %s %s %s %s %s %s \n",TAG_ID[0], TAG_ID[1], TAG_ID[2], TAG_ID[3], TAG_ID[4], TAG_ID[5]);
			Thread.sleep(1000);
	}

	public static void ReadTagData(int accessPassword) throws Exception
	{
		System.out.println("Read Tag Data");
		TagFields tagFields = new TagFields();
		TagMask tagMask = new TagMask();
		byte[] dataBanks = { 0, 0, 1, 2, 3 };
		short[] dataBankLengths = { 2,     // Kill Password
				2,     // Access Password
				8,     // EPC length
				6,     // TID length
				8 };   // User data length
		short[] dataBankAddress = { 0, 2, 0, 0, 0 };


		tagMask.maskBank = 2;
		tagMask.maskStartBitAddress = 0;
		tagMask.maskBitLength = 96;
		String[] tagIDTag = TAG_ID;



		for (int idx = 0; idx < tagMask.maskBitLength / 16; idx++)
		{
			short tidWord = 0;
			tidWord = (short)Integer.parseInt(tagIDTag[idx],16);
			tagMask.mask[idx * 2] = (byte)((tidWord >> 8) & 0xFF);
			tagMask.mask[idx * 2 + 1] = (byte)(tidWord & 0xFF);
		}

		for (int idx = 0; idx < dataBanks.length; idx++)
		{
			tagFields.errorCode = 0;
			tagFields.data = new short[100];
			tagFields.dataWordLength = dataBankLengths[idx];
			tagFields.dataStartWord = dataBankAddress [idx];

			String thePassword = "";

			tagFields.dataBank = dataBanks[idx];


			ReadWriteTags.reader.ReadTagData(accessPassword, tagMask, tagFields);

			switch (idx)
			{
			case 0:
				thePassword = String.format("%08X", tagFields.data[0] << 16 | tagFields.data[1]);
				System.out.println("Kill Password: " + thePassword);
				break;

			case 1:
				thePassword = String.format("%08X", tagFields.data[0] << 16 | tagFields.data[1]);
				System.out.println("Access Password: " + thePassword);
				break;

			case 2: 
				String epcWhole = "";
				for (int jdx = 2; jdx < tagFields.dataWordLength; jdx++)
				{
					String epcWord = String.format("%04X", tagFields.data[jdx]);

					epcWhole += epcWord;
				}
				System.out.println("EPC: " + epcWhole);
				break;

			case 3:
				String tidTag = "";

				for (int jdx = 0; jdx < tagFields.dataWordLength; jdx++)
				{
					String tidWord = String.format("%04X", tagFields.data[jdx]);
					tidTag += tidWord; 
				}
				System.out.println("Tag ID: "+ tidTag);
				break;

			case 4:
				String umWhole = "";
				for (int jdx = 0; jdx < tagFields.dataWordLength; jdx++)
				{
					String umWord = String.format("%04X", tagFields.data[jdx]);
					umWhole += umWord;
				}
				System.out.println("User Data: " + umWhole);
				break;
			}
		}
		Thread.sleep(1000);
	}
	

	
	public static void WriteTagData(int accessPassword) throws Exception
	{
		TagFields tagFields = new TagFields();
		TagMask tagMask = new TagMask();
		
		tagMask.maskBank = 2;
		tagMask.maskStartBitAddress = 0;
		tagMask.maskBitLength = 96;
		String[] tidTag = TAG_ID;
		int reattempts = 0;
		
		for (int idx = 0; idx < tagMask.maskBitLength / 16; idx++)
		{
			short tidWord = 0;
			tidWord = (short)Integer.parseInt(tidTag[idx],16);
			tagMask.mask[idx * 2] = (byte)((tidWord >> 8) & 0xFF);
			tagMask.mask[idx * 2 + 1] = (byte)(tidWord & 0xFF);
		}
				
		Byte[] dataBanks = { 0, 0, 1, 3 };
        short[] dataBankLengths = { 2,     // Kill Password
                                     2,     // Access Password
                                     6,     // EPC length
                                     8 };   // User data length
        short[] dataBankAddresses = { 0, 2, 2, 0 };
        
        for (int fieldIndex = 0; fieldIndex < dataBanks.length; fieldIndex++)
        {
        	tagFields.data = new short[100];
        	tagFields.dataBank = dataBanks[fieldIndex];
        	tagFields.dataStartWord = dataBankAddresses[fieldIndex];
        	tagFields.dataWordLength = dataBankLengths[fieldIndex];
        	tagFields.errorCode = 0;
        	
        	switch (fieldIndex)
        	{
        	case 0:
        		//Updating Kill Password
        		int killPassword = TAG_KILL_PASSWORD+1;
        		for (int idx = 0; idx < 2; idx++)
        		{
        			tagFields.data[idx] = (short)((killPassword >> (1-idx) * 16) & 0xFFFF);
        		}
        		break;
        		
        	case 1:
        		//Updating Access Password
        		int writeAccessPassword = accessPassword+1;
        		tagFields.errorCode = 0;
        		for (int idx = 0; idx < 2; idx++)
        			tagFields.data[idx] = (short)((writeAccessPassword >> (1 - idx) * 16) & 0xFFFF);
        		
        		break;
        		
        	case 2:
        		//Updating EPC
        		
        		String[] epcParts = {"5555","5555","5555","5555","5555","6666"};
        		tagFields.errorCode = 0;
        		for (int idx = 0; idx < tagFields.dataWordLength; idx++)
        		{
        			tagFields.data[idx] = (short)Integer.parseInt(epcParts[idx],16);
        		}
        		break;
        		
        	case 3:
        		//Updating user data
        		String[] userData = {"0000","0000","0000","0000","0000","0000","0000","0006"};
        		tagFields.errorCode = 0;
        		for (int idx = 0; idx < tagFields.dataWordLength; idx++)
        		{
        			tagFields.data[idx] = (short)Integer.parseInt(userData[idx],16);
        		}
        		break;	
        	}
        	
        	if (fieldIndex == 0)
        		System.out.print("\nKill update error: ");
        	else if (fieldIndex == 1)
        		System.out.print("Access update error: ");
        	else if (fieldIndex == 2)
        		System.out.print("EPC update error: ");
        	else if (fieldIndex == 3)
        		System.out.print("User update error: ");
        	
        	ReadWriteTags.reader.WriteTagData(accessPassword, tagMask, tagFields);
        	
        	System.out.println(tagFields.errorCode);
        	
        	if(tagFields.errorCode == 3 || tagFields.errorCode == 1 && reattempts <= 1 )
        	{
        		System.out.printf("error detected, reattempt %d\n", reattempts+1);
        		fieldIndex--;
        		reattempts++;
        	}
        	else
        		reattempts = 0;
        }
        System.out.println("Tag Data Written\n");
        Thread.sleep(1000);
	}
	
	public static void WriteTagData2(int accessPassword) throws Exception
	{
		TagFields tagFields = new TagFields();
		TagMask tagMask = new TagMask();
		
		tagMask.maskBank = 2;
		tagMask.maskStartBitAddress = 0;
		tagMask.maskBitLength = 96;
		String[] tidTag = TAG_ID;
		int reattempts = 0;
		
		for (int idx = 0; idx < tagMask.maskBitLength / 16; idx++)
		{
			short tidWord = 0;
			tidWord = (short)Integer.parseInt(tidTag[idx],16);
			tagMask.mask[idx * 2] = (byte)((tidWord >> 8) & 0xFF);
			tagMask.mask[idx * 2 + 1] = (byte)(tidWord & 0xFF);
		}
		
		Byte[] dataBanks = { 0, 0, 1, 3 };
        short[] dataBankLengths = { 2,     // Kill Password
                                     2,     // Access Password
                                     6,     // EPC length
                                     8 };   // User data length
        short[] dataBankAddresses = { 0, 2, 2, 0 };
        
        for (int fieldIndex = 0; fieldIndex < dataBanks.length; fieldIndex++)
        {
        	tagFields.data = new short[100];
        	tagFields.dataBank = dataBanks[fieldIndex];
        	tagFields.dataStartWord = dataBankAddresses[fieldIndex];
        	tagFields.dataWordLength = dataBankLengths[fieldIndex];
        	tagFields.errorCode = 0;
        	
        	switch (fieldIndex)
        	{
        	case 0:
        		//Updating Kill Password
        		int killPassword = TAG_KILL_PASSWORD;
        		for (int idx = 0; idx < 2; idx++)
        		{
        			tagFields.data[idx] = (short)((killPassword >> (1-idx) * 16) & 0xFFFF);
        		}
        		break;
        		
        	case 1:
        		//Updating Access Password
        		int writeAccessPassword = TAG_ACCESS_PASSWORD;
        		
        		for (int idx = 0; idx < 2; idx++)
        			tagFields.data[idx] = (short)((writeAccessPassword >> (1 - idx) * 16) & 0xFFFF);
        		break;
        		
        	case 2:
        		//Updating EPC
//        		byte[] epcBytes = new byte[tagFields.dataWordLength * 2];
        		String[] epcParts = TAG_EPC;
        		
        		for (int idx = 0; idx < tagFields.dataWordLength; idx++)
        		{
        			tagFields.data[idx] = (short)Integer.parseInt(epcParts[idx],16);
        		}
        		break;
        		
        	case 3:
        		//Updating user data
        		String[] userData = {"0000","0000","0000","0000","0000","0000","0000","0004"};
        		for (int idx = 0; idx < tagFields.dataWordLength; idx++)
        		{
        			tagFields.data[idx] = (short)Integer.parseInt(userData[idx],16);
        		}
        		break;	
        	}
        	
        	if (fieldIndex == 0)
        		System.out.print("\nKill update error: ");
        	else if (fieldIndex == 1)
        		System.out.print("Access update error: ");
        	else if (fieldIndex == 2)
        		System.out.print("EPC update error: ");
        	else if (fieldIndex == 3)
        		System.out.print("User update error: ");
        	
        	ReadWriteTags.reader.WriteTagData(accessPassword, tagMask, tagFields);
        	System.out.println(tagFields.errorCode);
        	if(tagFields.errorCode == 3 || tagFields.errorCode == 1 && reattempts <= 1 )
        	{
        		System.out.printf("error detected. Reattempt %d", reattempts+1);
        		fieldIndex--;
        		reattempts++;
        	}
        	else
        		reattempts = 0;
        		
        }
        
        System.out.println("\nTag Data Reset\n");
        Thread.sleep(1000);
	}

}
